use rusqlite::Connection;

pub trait DataSource<T> {
    fn get_ith<'a>(&'a self, i: usize) -> Option<&'a T>;
}

pub struct ArraySource<T> {
    words: Vec<T>
}

impl<T> DataSource<T> for ArraySource<T> {
    fn get_ith<'a>(&'a self, i: usize) -> Option<&'a T> {
        self.words.get(i)
    }
}

impl<T> ArraySource<T> {
    pub fn create(words: Vec<T>) -> Self {
        ArraySource {
            words
        }
    }
}

pub struct SqliteSource {
    conn: Connection
}

impl DataSource<String> for SqliteSource {
    fn get_ith<'a>(&'a self, i: usize) -> Option<&'a String> {
        let word: rusqlite::Result<String> = self.conn.query_row(
            &format!(
                "SELECT question
                 FROM questions
                 WHERE id={};", i),
            rusqlite::NO_PARAMS,
            |row| row.get(0));
        
        //word.ok().as_ref()
        None
    }
}

impl SqliteSource {
    pub fn open(db_name: &str) -> Result<Self, rusqlite::Error> {
        Ok(SqliteSource {
            conn: Connection::open(db_name)?
        })
    }
}

lazy_static! {
    //static ref default_source: Box<dyn DataSource<String>> =
    //    Box::new(SqliteSource::open("questions.db").unwrap_or(ArraySource::create(vec!["".to_owned()])))
}
