use actix::dev::{MessageResponse, ResponseChannel};
use actix::prelude::*;
use crate::server::server::GameConnection;
use crate::server::gamelobby::GameLobby;
use crate::websocket::*;

#[derive(Message)]
#[rtype(result = "Answer")]
pub struct JoinRequest {
    pub lobby_id: String,
    pub nick: String,
    pub p: Addr<GameConnection>
}

#[derive(Message)]
#[rtype(result = "Answer")]
pub struct CreateLobbyRequest {
    pub lobby_id: String, pub p: Addr<GameConnection>
}

#[derive(Message)]
#[rtype(result = "()")]
pub struct ReadyMsg(pub String);

#[derive(Message)]
#[rtype(result = "()")]
pub struct SubmitWordMsg {
    pub word: String,
    pub nick: String
}

#[derive(Message)]
#[rtype(result = "()")]
pub struct SubmitGuessMsg {
    pub guesses: Vec<(String, String)>,
    pub nick: String
}

#[derive(Message)]
#[rtype(result = "()")]
pub struct ResultMsg {
    pub results: RoundResultData
}

#[derive(Message)]
#[rtype(result = "()")]
pub struct NoSuchLobby(pub String);

#[derive(Message)]
#[rtype(result = "()")]
pub struct LobbyJoined(pub Addr<GameLobby>);

#[derive(Message)]
#[rtype(result = "()")]
pub struct GameUpdate{
    pub game_data: GameData
}

pub enum Answer {
    LobbyJoined(Addr<GameLobby>),
    LobbyCreated(Addr<GameLobby>),
    LobbyAlreadyExists,
    NoSuchLobby,
}


impl<A, M> MessageResponse<A, M> for Answer
where
    A: Actor,
    M: Message<Result = Answer>,
{
    fn handle<R: ResponseChannel<M>>(self, _: &mut A::Context, tx: Option<R>) {
        if let Some(tx) = tx {
            tx.send(self);
        }
    }
}
